// Reverb model declaration
//
// Written by Jezar at Dreampoint, June 2000
// http://www.dreampoint.co.uk
// This code is public domain
// Modified for to use with TKS by Gerd Feldkirch (mailto:gerd.feldkirch@t-online.de), December 2003

#ifndef __FREEVERB_H__
#define __FREEVERB_H__

#include "comb.h"
#include "allpass.h"
#include "tuning.h"

class Freeverb : public YAC_Object
{
public:
	Freeverb();
	YAC_H( Freeverb );

	void mute();
	void processmix(float *inputL, float *inputR, float *outputL, float *outputR, long numsamples, int skip);
	void processreplace(float *inputL, float *inputR, float *outputL, float *outputR, long numsamples, int skip);
	void setroomsize(float value);
	float getroomsize();
	void setdamp(float value);
	float getdamp();
	void setwet(float value);
	float getwet();
	void setdry(float value);
	float getdry();
	void setwidth(float value);
	float getwidth();
	void setmode(float value);
	float getmode();

#define YAC_CLASS Freeverb
#define YAC_NUM_COMMANDS 15

#define YAC_CMD_0_NAME "operator"
#define YAC_CMD_0_RTTI 13
#define YAC_CMD_0_RET 4
#define YAC_CMD_0_SYM yacOperator

#define YAC_CMD_1_NAME "mute"
#define YAC_CMD_1_RTTI 0
#define YAC_CMD_1_SYM tksinter_mute
  void tksinter_mute( void );

#define YAC_CMD_2_NAME "processMix"
#define YAC_CMD_2_RTTI 15
#define YAC_CMD_2_SYM tksinter_processmix
  void tksinter_processmix( YAC_Object *_fa_source, YAC_Object *_fa_dest );

#define YAC_CMD_3_NAME "processReplace"
#define YAC_CMD_3_RTTI 15
#define YAC_CMD_3_SYM tksinter_processreplace
  void tksinter_processreplace( YAC_Object *_fa_source, YAC_Object *_fa_dest );

#define YAC_CMD_4_NAME "setroomSize"
#define YAC_CMD_4_RTTI 2
#define YAC_CMD_4_SYM tksinter_setroomsize
  void tksinter_setroomsize( sF32 _roomsize );
	
#define YAC_CMD_5_NAME "getRoomSize"
#define YAC_CMD_5_RTTI 0
#define YAC_CMD_5_RET 2
#define YAC_CMD_5_SYM tksinter_getroomsize
  void tksinter_getroomsize( YAC_Value *_r );

#define YAC_CMD_6_NAME "setDamping"
#define YAC_CMD_6_RTTI 2
#define YAC_CMD_6_SYM tksinter_setdamp
  void tksinter_setdamp( sF32 _damp );
	
#define YAC_CMD_7_NAME "getDamping"
#define YAC_CMD_7_RTTI 0
#define YAC_CMD_7_RET 2
#define YAC_CMD_7_SYM tksinter_getdamp
  void tksinter_getdamp( YAC_Value *_r );

#define YAC_CMD_8_NAME "setWet"
#define YAC_CMD_8_RTTI 2
#define YAC_CMD_8_SYM tksinter_setwet
  void tksinter_setwet( sF32 _wet );
	
#define YAC_CMD_9_NAME "getWet"
#define YAC_CMD_9_RTTI 0
#define YAC_CMD_9_RET 2
#define YAC_CMD_9_SYM tksinter_getwet
  void tksinter_getwet( YAC_Value *_r );

#define YAC_CMD_10_NAME "setDry"
#define YAC_CMD_10_RTTI 2
#define YAC_CMD_10_SYM tksinter_setdry
  void tksinter_setdry( sF32 _dry );
	
#define YAC_CMD_11_NAME "getDry"
#define YAC_CMD_11_RTTI 0
#define YAC_CMD_11_RET 2
#define YAC_CMD_11_SYM tksinter_getdry
  void tksinter_getdry( YAC_Value *_r );

#define YAC_CMD_12_NAME "setWidth"
#define YAC_CMD_12_RTTI 2
#define YAC_CMD_12_SYM tksinter_setwidth
  void tksinter_setwidth( sF32 _width );
	
#define YAC_CMD_13_NAME "getWidth"
#define YAC_CMD_13_RTTI 0
#define YAC_CMD_13_RET 2
#define YAC_CMD_13_SYM tksinter_getwidth
  void tksinter_getwidth( YAC_Value *_r );

#define YAC_CMD_14_NAME "setMode"
#define YAC_CMD_14_RTTI 2
#define YAC_CMD_14_SYM tksinter_setmode
  void tksinter_setmode( sF32 _width );
	
#define YAC_CMD_15_NAME "getMode"
#define YAC_CMD_15_RTTI 0
#define YAC_CMD_15_RET 2
#define YAC_CMD_15_SYM tksinter_getmode
  void tksinter_getmode( YAC_Value *_r );
	
#include <yac_createcommands.h>
	
private:
	void	update();
private:
	float	gain;
	float	roomsize,roomsize1;
	float	damp,damp1;
	float	wet,wet1,wet2;
	float	dry;
	float	width;
	float	mode;

	// The following are all declared inline 
	// to remove the need for dynamic allocation
	// with its subsequent error-checking messiness

	// Comb filters
	comb	combL[numcombs];
	comb	combR[numcombs];

	// Allpass filters
	allpass	allpassL[numallpasses];
	allpass	allpassR[numallpasses];

	// Buffers for the combs
	float	bufcombL1[combtuningL1];
	float	bufcombR1[combtuningR1];
	float	bufcombL2[combtuningL2];
	float	bufcombR2[combtuningR2];
	float	bufcombL3[combtuningL3];
	float	bufcombR3[combtuningR3];
	float	bufcombL4[combtuningL4];
	float	bufcombR4[combtuningR4];
	float	bufcombL5[combtuningL5];
	float	bufcombR5[combtuningR5];
	float	bufcombL6[combtuningL6];
	float	bufcombR6[combtuningR6];
	float	bufcombL7[combtuningL7];
	float	bufcombR7[combtuningR7];
	float	bufcombL8[combtuningL8];
	float	bufcombR8[combtuningR8];

	// Buffers for the allpasses
	float	bufallpassL1[allpasstuningL1];
	float	bufallpassR1[allpasstuningR1];
	float	bufallpassL2[allpasstuningL2];
	float	bufallpassR2[allpasstuningR2];
	float	bufallpassL3[allpasstuningL3];
	float	bufallpassR3[allpasstuningR3];
	float	bufallpassL4[allpasstuningL4];
	float	bufallpassR4[allpasstuningR4];
};

#endif /* __FREEVERB_H__ */
